/**
*************************************************************************************************************
*
*   \file nmeaconv.h
*
*   NMEA Data conversion functions
*
*   Module contains all conversion functions which are used to convert and store the
*	data of the different NMEA frames. These functions may be modified according to
*	the system needs. The type of these functions is fixed.\n
*   \n
*   \b Shortcut:    NCONV
*
*   \author         Ralf Hochhausen
*   \date           23.06.2006
*   \version        1.0
*
*   (c) 2006 by Ralf Hochhausen (e-mail: micro@ralf-hochhausen.de)
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
*   \b History:
*
*   23.06.2006      Ralf Hochhausen\n
*                   Revision 1.0\n
*                   Initial Revision
*
*************************************************************************************************************
*/

#ifndef __NCONV_H
#define __NCONV_H
/*
*************************************************************************************************************
*   INCLUDES 
*************************************************************************************************************
*/
#include "types.h"				/* Include basic types */

/*
*************************************************************************************************************
*   GLOBAL DEFINITIIONS
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   GLOBAL MACROS
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   GLOBAL TYPES
*************************************************************************************************************
*/

/**
*	Type of the NMEA data conversion functions
*/
typedef uchar8 tNMEADataConverter(uchar8 *pucData_, void *pTargetVar_);

/**
*	Pointertype to address data conversion functions
*/
typedef tNMEADataConverter *tfpConvFunction;

/**
*	Enumeration of the provided data types
*
*	The entries in this must have the same order like the entries in the
*	converter function table!
*/
typedef enum NMEADataType
{
	NCONV_TypeLatitude,			/**< Datatype Latitude */
	NCONV_TypeLongitude,		/**< Datatype Longitude */
	NCONV_TypeTime,				/**< Datatype Time */
	NCONV_TypeStatus,			/**< Datatype Status */
	NCONV_TypeVariable,			/**< Datatype Variable */
	NCONV_TypeFixedHEX,			/**< Datatype Fiexed HEX */
	NCONV_TypeFixedAlpha,		/**< Datatype Fixed Alpha */
	NCONV_TypeFixedNumber,		/**< Datatype Fixed Number */
	NCONV_TypeUndefined			/**< Datatype Undefined (data that is not parsed) */
}tNMEADataType;

/**
*	Data connection type
*
*	Functions need to be connected to a certain variable which is used to store the
*	conversion results. All connections need to have this type.
*/
typedef struct NMEADataConnector
{
	tNMEADataType	eDataType;	/**< Connection to the converter function */
	void*			pTargetVar;	/**< Connection to the target variable */
}tNMEADataConnector;

/*
*************************************************************************************************************
*   GLOBAL CONSTANTS
*************************************************************************************************************
*/

/* Table with the different NMEA data conversion functions*/
extern const tfpConvFunction NCONV_fpConvFunctionTable[] PROGMEM;

/*
*************************************************************************************************************
*   GLOBAL FUNCTIONS
*************************************************************************************************************
*/

/**
*************************************************************************************************************
*
*   Convert latitude datafield
*
*   Function copies the latitude into the target variable
*
*   \param
*		pucData_    Pointer to the data field within the NMEA frame
*   \param
*		pTargetVar_	Pointer to the target variable where the converted data has to be stored
*
*   \return
*		uchar8		Actually not supported
*   
*************************************************************************************************************
*/
uchar8 NCONV_ucConvertLatitude(uchar8 *pucData_, void *pTargetVar_);

/**
*************************************************************************************************************
*
*   Convert longitude datafield
*
*   Function copies the longitude into the target variable
*
*   \param
*		pucData_    Pointer to the data field within the NMEA frame
*   \param
*		pTargetVar_	Pointer to the target variable where the converted data has to be stored
*
*   \return
*		uchar8		Actually not supported
*   
*************************************************************************************************************
*/
uchar8 NCONV_ucConvertLongitude(uchar8 *pucData_, void *pTargetVar_);

/**
*************************************************************************************************************
*
*   Convert time datafield
*
*   Function copies the time into the target variable
*
*   \param
*		pucData_    Pointer to the data field within the NMEA frame
*   \param
*		pTargetVar_	Pointer to the target variable where the converted data has to be stored
*
*   \return
*		uchar8		Actually not supported
*   
*************************************************************************************************************
*/
uchar8 NCONV_ucConvertTime(uchar8 *pucData_, void *pTargetVar_);

/**
*************************************************************************************************************
*
*   Convert status datafield
*
*   Function copies the status into the target variable
*
*   \param
*		pucData_    Pointer to the data field within the NMEA frame
*   \param
*		pTargetVar_	Pointer to the target variable where the converted data has to be stored
*
*   \return
*		uchar8		Actually not supported
*   
*************************************************************************************************************
*/
uchar8 NCONV_ucConvertStatus(uchar8 *pucData_, void *pTargetVar_);

/**
*************************************************************************************************************
*
*   Convert variable datafield
*
*   Function copies the variable into the target variable
*
*   \param
*		pucData_    Pointer to the data field within the NMEA frame
*   \param
*		pTargetVar_	Pointer to the target variable where the converted data has to be stored
*
*   \return
*		uchar8		Actually not supported
*   
*************************************************************************************************************
*/
uchar8 NCONV_ucConvertVariable(uchar8 *pucData_, void *pTargetVar_);

/**
*************************************************************************************************************
*
*   Convert fixed hex datafield
*
*   Function copies the fixed hex number into the target variable
*
*   \param
*		pucData_    Pointer to the data field within the NMEA frame
*   \param
*		pTargetVar_	Pointer to the target variable where the converted data has to be stored
*
*   \return
*		uchar8		Actually not supported
*   
*************************************************************************************************************
*/
uchar8 NCONV_ucConvertFixedHEX(uchar8 *pucData_, void *pTargetVar_);

/**
*************************************************************************************************************
*
*   Convert fixed alpha datafield
*
*   Function copies the fixed alpha into the target variable
*
*   \param
*		pucData_    Pointer to the data field within the NMEA frame
*   \param
*		pTargetVar_	Pointer to the target variable where the converted data has to be stored
*
*   \return
*		uchar8		Actually not supported
*   
*************************************************************************************************************
*/
uchar8 NCONV_ucConvertFixedAlpha(uchar8 *pucData_, void *pTargetVar_);

/**
*************************************************************************************************************
*
*   Convert fixed number datafield
*
*   Function copies the fixed number into the target variable
*
*   \param
*		pucData_    Pointer to the data field within the NMEA frame
*   \param
*		pTargetVar_	Pointer to the target variable where the converted data has to be stored
*
*   \return
*		uchar8		Actually not supported
*   
*************************************************************************************************************
*/
uchar8 NCONV_ucConvertFixedNumber(uchar8 *pucData_, void *pTargetVar_);

#endif  /* __NCONV_H */
