/**
*************************************************************************************************************
*
*   \file nmeadll.c
*
*   NMEA data link layer.
*
*	Detects NMEA frames and writes them into the
*	internal buffer. Checksum is calculated. If checksum is wrong the
*	received frame is discarded. In case of a correct received frame
*	an event is generated to trigger frame parsing.\n
*
*   \n
*   \b Shortcut:    NDLL
*
*   \author         Ralf Hochhausen
*   \date           15.06.2006
*   \version        1.0
*
*   (c) 2006 by Ralf Hochhausen (e-mail: micro@ralf-hochhausen.de)
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
*   \b History:
*
*   15.06.2006      Ralf Hochhausen\n
*                   Revision 1.0\n
*                   Initial Revision
*
*************************************************************************************************************
*/

/**
*	\todo	Change overflow handling. When overflow occures new NMEA sentences are ignored
*/

/*
*************************************************************************************************************
*   INCLUDES 
*************************************************************************************************************
*/

#include <avr\io.h>				/* Include IO register definitions */
#include <avr\interrupt.h>		/* Include interrupt/signal definitions */
#include "compile.h"			/* Include compiler switches */
#include "types.h"				/* Include basic types */
#include "global.h"				/* Include global makros */
#include "sysparam.h"			/* Include global system parameters */
#include "timer.h"				/* Include SW-timers for timeout detection */
#include "utility.h"			/* Include generic utilities */
#include "nmeadll.h"			/* Include own header */

/*
*************************************************************************************************************
*   LOCAL DEFINITIIONS
*************************************************************************************************************
*/
/* UART configurations */
#define NDLL_BAUDRATE		4800L			/**< Baudrate of the asynchronous interface */
#define	NDLL_SPEEDMODE		16				/**< Interface operates in normal mode */
#define NDLL_UBRR			(SYSP_FCPU/(NDLL_SPEEDMODE*NDLL_BAUDRATE)-1) /**< Baudrate cfg value */

/* Timing definitions */
#define NDLL_TIMEOUT_TIME_10MS		10		/**< Timeout is generated after 100ms during reception */

/* Definition of the receive buffer */
#define NDLL_MAX_NMEA_FRAME_LENGTH	77		/**< Max. NMEA frame length excl. '$' and CR/LF */
#define NDLL_RECEIVE_BUFFERSIZE		4		/**< Buffersize of the receive buffer (4 frames of max. size) */

/* Definition of NMEA specific characters */
#define NDLL_NMEA_SOF_SIGN			'$'		/**< NMEA start of frame sign */
#define NDLL_NMEA_CHECKSUM_DEL		'*'		/**< NMEA checksum delimiter */

/*
*************************************************************************************************************
*   LOCAL MACROS
*************************************************************************************************************
*/
#define NDLL_mInitChecksum()	(NDLL_ucChecksum = 0)	/**< Reset checksum calculation */

/*
*************************************************************************************************************
*   LOCAL TYPES
*************************************************************************************************************
*/
/** enumeration of the NDLL states */
typedef enum NMEADLLState
{
	NDLL_STATE_WAIT_FOR_SOF,					/**< Wait for start of NMEA frame ('$'-sign) */
	NDLL_STATE_READ_FRAME,						/**< Write frame into buffer an calculate checksum */
	NDLL_STATE_WAIT_FOR_CHECKSUM_HIGH_NIBBLE,	/**< Wait for checksum high nibble */
	NDLL_STATE_WAIT_FOR_CHECKSUM_LOW_NIBBLE,	/**< Wait for checksum low nibble */
	NDLL_STATE_WAIT_FOR_EOF_CR,					/**< Wait for end of frame (CR) */
	NDLL_STATE_WAIT_FOR_EOF_LF,					/**< Wait for end of frame (LF) */
}tNMEADLLState;

/** enumeration of the NDLL receive buffer states */
typedef enum BufferStates
{
	NDLL_BUFFER_STATE_OK,						/**< Buffer is ready to store NMEA frames */
	NDLL_BUFFER_STATE_FULL = 0x01				/**< Receive buffer is full */
}tBufferStates;

/*
*************************************************************************************************************
*   LOCAL CONSTANTS
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   STATIC VARIABLES
*************************************************************************************************************
*/
static tNMEADLLState NDLL_eState;			/**< State of the NMEA DLL statemachine */
static uchar8 NDLL_ucChecksum;				/**< Variable for checksum generation */
static uchar8 NDLL_aucReceiveBuffer[NDLL_RECEIVE_BUFFERSIZE][NDLL_MAX_NMEA_FRAME_LENGTH];/**< NMEA receive buffer */
static uchar8 NDLL_ucNextFrameToProcess;	/**< Index of the next frame to process */
static uchar8 NDLL_ucNMEAFramesInBuffer;	/**< Number of frames in buffer */
static uchar8 NDLL_ucNextFreeFrameBuffer;	/**< Next free frame buffer */
static uchar8 NDLL_ucBufferWriteIdx;		/**< Index to the next free buffer cell */
static uchar8 NDLL_ucBufferState;			/**< Flag register of the receive buffer */

static tEventTimer NDLL_TmrTimeOut;			/**< Timer which is used for timeout detection */

#if NDLL_DBG_NMEA_FRAME == OPTION_ACTIVE

static uchar8 DBG_aucNMEAFrame[] = "GPGGA,164432.00,4845.1203,N,01125.8512,E,1,03,3.27,00354,M,047,M,,";

#endif

/*
*************************************************************************************************************
*   LOCAL FUNCTION PROTOTYPES
*************************************************************************************************************
*/

void vAddByteToChecksum(uchar8 ucByte_);
uchar8 ucConvNibbleToAscii(uchar8 ucByte_);
void vTimeoutEvent(void);
void vResetInterface(void);

/*
*************************************************************************************************************
*   LOCAL FUNCTIONS
*************************************************************************************************************
*/

/**
*************************************************************************************************************
*	\brief	UART0 receive complete interrupt
*
*	Interrupt is called on every reception of a NMEA byte. Depending on the actual
*	state the bytes are buffered and added to the checksum. If a complete NMEA frame
*	has been received correctly, the frame can be processed by an upper layer. If the
*	frame was received erroneous it is released from buffer.
*
*************************************************************************************************************
*/
ISR(SIG_UART0_RECV)
{
	uchar8 ucReceivedByte_;		/* Temporary variable for reading UDR0 */

	ucReceivedByte_ = UDR0;

	if(NDLL_ucBufferState &= NDLL_BUFFER_STATE_FULL)
	{
		return;		/* Buffer is full => data must not be stored */
	}
	else
	{
		
#if NDLL_DEBUG_COM == OPTION_ACTIVE

		UDR0 = ucReceivedByte_;	/* Return received byte */

#endif
	
		/* New byte has been received => reload timeout timer */
		TMR_vLoadEventTimer(&NDLL_TmrTimeOut, NDLL_TIMEOUT_TIME_10MS, vTimeoutEvent);
	
		switch (NDLL_eState)
		{
	
		case NDLL_STATE_WAIT_FOR_SOF:
			/* Wait for start of frame sign*/
			if(ucReceivedByte_ == NDLL_NMEA_SOF_SIGN)
			{
				/* Start of NMEA frame detected write frame into buffer */
				NDLL_mInitChecksum();
				NDLL_eState = NDLL_STATE_READ_FRAME;
			}
			break;

		case NDLL_STATE_READ_FRAME:
			/* Write Frame into buffer until checksum delimiter has been detected */
			if(ucReceivedByte_ == NDLL_NMEA_CHECKSUM_DEL)
			{
				/* Checksum delimiter has been detected => next byte is checksum high nibble */
				if(NDLL_ucBufferWriteIdx < NDLL_MAX_NMEA_FRAME_LENGTH)
				{
					/* Finish frame with \0 character */
					NDLL_aucReceiveBuffer[NDLL_ucNextFreeFrameBuffer][NDLL_ucBufferWriteIdx] = 0;
					NDLL_eState = NDLL_STATE_WAIT_FOR_CHECKSUM_HIGH_NIBBLE;
				}
				else
				{
					/* Overflow has occured => reset reception */
					vResetInterface();
				}
			}
			else
			{
				/* Store received byte in buffer and add it to checksum */
				if(NDLL_ucBufferWriteIdx < NDLL_MAX_NMEA_FRAME_LENGTH)
				{
					NDLL_aucReceiveBuffer[NDLL_ucNextFreeFrameBuffer][NDLL_ucBufferWriteIdx] = ucReceivedByte_;
					vAddByteToChecksum(ucReceivedByte_);
					NDLL_ucBufferWriteIdx++;
				}
				else
				{
					/* Overflow has occured => reset reception */
					vResetInterface();
				}
			
			}

			break;

		case NDLL_STATE_WAIT_FOR_CHECKSUM_HIGH_NIBBLE:
			if(ucReceivedByte_ == UTI_ucConvHexNumberToAscii(UTI_mGetHighNibble(NDLL_ucChecksum)))
			{
				/* High nibble of checksum is ok => wait for low nibble */
				NDLL_eState = NDLL_STATE_WAIT_FOR_CHECKSUM_LOW_NIBBLE;
			}
			else
			{
				/* Checksum was not ok => reset reception */
				vResetInterface();
			}
			break;

		case NDLL_STATE_WAIT_FOR_CHECKSUM_LOW_NIBBLE:

			if(ucReceivedByte_ == UTI_ucConvHexNumberToAscii(UTI_mGetLowNibble(NDLL_ucChecksum)))
			{
				/* Checksum is ok => wait for end of frame */
				NDLL_eState = NDLL_STATE_WAIT_FOR_EOF_CR;
			}
			else
			{
				/* Checksum was not ok => reset reception */
				vResetInterface();
			}		
		
			break;

		case NDLL_STATE_WAIT_FOR_EOF_CR:
			if(ucReceivedByte_ == 0x0D)
			{
				NDLL_eState = NDLL_STATE_WAIT_FOR_EOF_LF;
			}
			else
			{
				/* Received byte was not expected => reset reception */
				vResetInterface();		
			}
			break;

		case NDLL_STATE_WAIT_FOR_EOF_LF:
			if(ucReceivedByte_ == 0x0A)
			{
				/* Received frame was ok and can be accepted => increment indexes */
				
				NDLL_ucNMEAFramesInBuffer++;
				if(NDLL_ucNMEAFramesInBuffer == NDLL_RECEIVE_BUFFERSIZE)
				{
					/* Receive buffer is full => no more NMEA frames can be stored until
					   at least one frame is processed */
					NDLL_ucBufferState |= NDLL_BUFFER_STATE_FULL;
				}
			
				NDLL_ucNextFreeFrameBuffer++;
				if (NDLL_ucNextFreeFrameBuffer >= NDLL_RECEIVE_BUFFERSIZE)
				{
					/* End of ringbuffer has been reached => restart from the beginning */
					NDLL_ucNextFreeFrameBuffer = 0;
				}
			}
			else
			{
				/* Received byte was not expected => only reset reception */
			}
			vResetInterface();
			break;
		}
	}
}

/**
*************************************************************************************************************
*	\brief	Calculation of the NMEA checksum
*
*	Function calculates the NMEA checksum. It needs to be called after reception of
*	each byte. 
*	
*	\return	
*		void
*
*	\param		
*		ucByte_		Byte to add to checksum
*	
*************************************************************************************************************
*/
void vAddByteToChecksum(uchar8 ucByte_)
{
	NDLL_ucChecksum ^= ucByte_;
}

/**
*************************************************************************************************************
*	\brief	Reset NMEA reception statemachine
*
*	Function is called in case of any timeout or unexpected character reception.
*	
*	\return	
*		void
*	
*************************************************************************************************************
*/
void vResetInterface(void)
{
	/* Reception timeout has been detected => reset reception */
	vEnterCritical();			/* Variables have to be written without interruption */
	NDLL_eState = NDLL_STATE_WAIT_FOR_SOF;
	NDLL_ucBufferWriteIdx = 0;
	vExitCritical();
}

/**
*************************************************************************************************************
*	\brief	Timeout event function
*
*	Function is called in case of any reception timeout. When a timeout is detected
*	the receive statemachine is reset.
*	
*	\return		void
*	
*************************************************************************************************************
*/
void vTimeoutEvent(void)
{
	vResetInterface();
}

/*
*************************************************************************************************************
*   GLOBAL FUNCTIONS
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   uchar8 *NDLL_pucGetReceivedNMEAFrame(void)
*************************************************************************************************************
*/
uchar8 *NDLL_pucGetReceivedNMEAFrame(void)
{
#if NDLL_DBG_NMEA_FRAME == OPTION_ACTIVE

	return (DBG_aucNMEAFrame);
#else

	return (&NDLL_aucReceiveBuffer[NDLL_ucNextFrameToProcess][0]);

#endif
}

/*
*************************************************************************************************************
*   void NDLL_vReleaseFrameFromBuffer(void)
*************************************************************************************************************
*/
void NDLL_vReleaseFrameFromBuffer(void)
{
#if NDLL_DBG_NMEA_FRAME == OPTION_ACTIVE
#else

	/* Decrement only if any frame has been in buffer */
	if(NDLL_ucNMEAFramesInBuffer)
	{
		vEnterCritical();				/* The following variables are also used in interrupts! */
		NDLL_ucNMEAFramesInBuffer--;
		NDLL_ucBufferState &= ~NDLL_BUFFER_STATE_FULL;	/* At least one NMEA frame can be stored */
		vExitCritical();

		/* Set pointer on the next frame */
		NDLL_ucNextFrameToProcess++;
		if (NDLL_ucNextFrameToProcess >= NDLL_RECEIVE_BUFFERSIZE)
		{
			/* End of ringbuffer has been reached => restart from the beginning */
			NDLL_ucNextFrameToProcess = 0;
		}
	}

#endif
}

/*
*************************************************************************************************************
*   uchar8 NDLL_ucIsFrameInBuffer(void)
*************************************************************************************************************
*/
uchar8 NDLL_ucIsFrameInBuffer(void)
{
#if NDLL_DBG_NMEA_FRAME == OPTION_ACTIVE

	return TRUE;

#else

	if(NDLL_ucNMEAFramesInBuffer)
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}

#endif
}

/*
*************************************************************************************************************
*   void NDLL_vHandleTask(void)
*************************************************************************************************************
*/
void NDLL_vHandleTask(void)
{
	switch (NDLL_eState)
	{
	
	case NDLL_STATE_WAIT_FOR_SOF:
		TMR_vLoadEventTimer(&NDLL_TmrTimeOut, NDLL_TIMEOUT_TIME_10MS, vTimeoutEvent);
		break;
	
	case NDLL_STATE_READ_FRAME:
	case NDLL_STATE_WAIT_FOR_CHECKSUM_LOW_NIBBLE:
	case NDLL_STATE_WAIT_FOR_CHECKSUM_HIGH_NIBBLE:
	case NDLL_STATE_WAIT_FOR_EOF_CR:
	case NDLL_STATE_WAIT_FOR_EOF_LF:
	default:
		break;

	}
}

/*
*************************************************************************************************************
*   void NDLL_vInit(void)
*************************************************************************************************************
*/
void NDLL_vInit(void)
{
	/* Initialize serial communication interface */

	/**
	*	\todo	move serial interface to a seperate module
	*/
	UCSR0A = 0x00;			/* USART operates at normal speed */
							/* Multi-Processor mode is disabled */

	UCSR0B = (	(1<<RXCIE) |	/* Enable receive complete interrupt */
				(1<<RXEN));		/* Enable receiver */

#if NDLL_DEBUG_COM == OPTION_ACTIVE

	UCSR0B |= (1<<TXEN);	/* Enable transmitter */

#endif

	UCSR0C = (	(0<<UMSEL) |	/* Configure for asynchronous operation */
				(0<<UPM1)  |	/* No parity */
				(0<<UPM0)  |
				(0<<USBS)  |	/* 1 stopbit */
				(1<<UCSZ1) |	/* 8 data bits */
				(1<<UCSZ0));

	UBRR0H = UTI_mGetHighByte(NDLL_UBRR);	/* Configure Baudrate Register */
	UBRR0L = UTI_mGetLowByte(NDLL_UBRR);


	/* Initialize all static variables */

	NDLL_eState = NDLL_STATE_WAIT_FOR_SOF;	/* Start waiting for first frame */
	NDLL_mInitChecksum();
	NDLL_ucNextFrameToProcess = 0;
	NDLL_ucNMEAFramesInBuffer = 0;
	NDLL_ucNextFreeFrameBuffer = 0;
	NDLL_ucBufferWriteIdx = 0;
	NDLL_ucBufferState = NDLL_BUFFER_STATE_OK;

	TMR_vCreateEventTimer(&NDLL_TmrTimeOut);	/* Create timeout timer */
	TMR_vLoadEventTimer(&NDLL_TmrTimeOut, NDLL_TIMEOUT_TIME_10MS, vTimeoutEvent);
}
