/**
*************************************************************************************************************
*
*   \file nmeaparser.c
*
*   NMEA protocol parser
*
*   Module provides functions that parse the NMEA protocol.\n
*   \n
*   \b Shortcut:    NPAR
*
*   \author         Ralf Hochhausen
*   \date           23.06.2006
*   \version        1.0
*
*   (c) 2006 by Ralf Hochhausen (e-mail: micro@ralf-hochhausen.de)
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
*   \b History:
*
*   23.06.2006      Ralf Hochhausen\n
*                   Revision 1.0\n
*                   Initial Revision
*
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   INCLUDES 
*************************************************************************************************************
*/
#include <avr/pgmspace.h>		/* Include ROM access functions */
#include "types.h"				/* Include basic types */
#include "nmeadll.h"			/* Include NMEA data link layer */
#include "nmeaframecfg.h"		/* Include NMEA frame configuration */
#include "nmeaconv.h"			/* Include NMEA data conversion functions */
#include "nmeaparser.h"			/* Include own header */

/*
*************************************************************************************************************
*   LOCAL DEFINITIIONS
*************************************************************************************************************
*/
#define NPAR_FRAME_NOT_SUPPORTED	0xFF	/**< Return value for unsupported NMEA frames */
#define NPAR_MSGID_LENGTH			5		/**< Message ID is 5 characters long */
#define NPAR_DATA_FIELD_SEPARATOR	','		/**< Data field separator */
#define NPAR_END_OF_NMEA_FRAME		0		/**< All NMEA frames end with \0 in buffer */

/*
*************************************************************************************************************
*   LOCAL MACROS
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   LOCAL TYPES
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   LOCAL CONSTANTS
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   STATIC VARIABLES
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   LOCAL FUNCTION PROTOTYPES
*************************************************************************************************************
*/
uchar8 ucIsFrameSupported(uchar8 *pucMessage_);
void vParseFrame(uchar8 *pucMessage_, uchar8 ucFrameNo_);

/*
*************************************************************************************************************
*   LOCAL FUNCTIONS
*************************************************************************************************************
*/

/**
*************************************************************************************************************
*	\brief	Check NMEA message identifier
*
*	The function checks if the message identifier of the two given messages are equal.
*	The first message is the received NMEA message which comes from the GPS receiver,
*	the second message is an entry in the NMEA configuration table.\n
*	The message identifier is located in the first five bytes of the received NMEA
*	message. This identifier can be devided into two main parts:\n
*	\li	The first two bytes represent the talker identification. GP signifies a GPS receiver.
*	\li The last three bytes represent the transmitted message
*	
*	\return	
*		BOOL	TRUE	=> Identifiers are equal\n
*				FALSE	=> Identifiers are not equal
*
*	\param		
*		pucMessage_	Pointer to the received NMEA frame
*
*	\param		
*		psFrame_	Pointer to a frame in the NMEA configuration table
*
*************************************************************************************************************
*/
BOOL bCheckMsgID(uchar8 *pucMessage_, const tNMEAFrameFormat *psFrame_)
{
	uchar8	ucIdx_;
	uchar8	ucCharacter_;
	BOOL	bReturnValue_ = TRUE;
	
	for(ucIdx_ = 0; ucIdx_ < NPAR_MSGID_LENGTH; ucIdx_++)
	{
		ucCharacter_ = (uchar8)pgm_read_byte(&(psFrame_->ucMSG[ucIdx_]));
		
		if(ucCharacter_ != pucMessage_[ucIdx_])
		{
			bReturnValue_ = FALSE;
			break;
		}
	}

	return bReturnValue_;
}

/**
*************************************************************************************************************
*	\brief	Check if the received NMEA frame is supported by the parser
*
*	The function walks through the NMEA configuration table and checks if the received
*	NMEA message corresponds to an entry in this table. If the message has an entry
*	it can be parsed by this parser module.
*	
*	\return	
*		uchar8		Index of the NMEA frame in the configuration table if frame is supported or
*					NPAR_FRAME_NOT_SUPPORTED if the frame is not supported
*
*	\param
*		pucMessage_	Pointer to the received NMEA frame		
*	
*************************************************************************************************************
*/
uchar8 ucIsFrameSupported(uchar8 *pucMessage_)
{
	uchar8 ucFrameIdx_;
	const tNMEAFrameFormat *psFrame_;
	uchar8 ucReturnValue_ = NPAR_FRAME_NOT_SUPPORTED;

	for(ucFrameIdx_ = 0; ucFrameIdx_ < NCFG_SUPPORTED_FRAME_NO; ucFrameIdx_++)
	{
		psFrame_ = (tNMEAFrameFormat*)pgm_read_word(&NCFG_asSupportedFrames[ucFrameIdx_]);

		if(bCheckMsgID(pucMessage_, psFrame_) == TRUE)
		{
			ucReturnValue_ = ucFrameIdx_;
			break;
		}
	}

	return ucReturnValue_;
}

/**
*************************************************************************************************************
*	\brief	Check NMEA frame
*
*	Function counts the number of datafields within the current NMEA frame which has
*	to be parsed. Datafields are separated with a ','-sign. If the number of datafields
*	in the current frame is not equal with the configuration, the frame is assumed
*	to be invalid.
*
*	\param
*		pucMessage_		Pointer to the NMEA frame to parse
*
*	\param
*		ucFrameNo_		Number of the current NMEA frame in the configuration table
*
*	\return
*		BOOL			TRUE	=> NMEA frame is valid
*						FALSE	=> NMEA frame is not valid
*
*************************************************************************************************************
*/
BOOL bCheckNMEAFrame(uchar8 *pucMessage_, uchar8 ucFrameNo_)
{
	const	tNMEAFrameFormat *psFrameCfgEntry_;
	uchar8	ucNoOfCfgDataFields_;
	uchar8	ucIdx_ = 0;
	uchar8	ucCtr_ = 0;

	/* Read pointer to the NMEA frame configuration */
	psFrameCfgEntry_ = (tNMEAFrameFormat*)pgm_read_word(&NCFG_asSupportedFrames[ucFrameNo_]);

	ucNoOfCfgDataFields_ = (uchar8)pgm_read_byte(&(psFrameCfgEntry_->ucNoOfDataFields));

	/* Count number of datafields in the current NMEA frame */
	while(pucMessage_[ucIdx_] != NPAR_END_OF_NMEA_FRAME)
	{
		if(pucMessage_[ucIdx_] == NPAR_DATA_FIELD_SEPARATOR)
		{
			ucCtr_++;
		}
		
		ucIdx_++;
	}

	/* Check if real number of datafields is equal to the configuration */
	if(ucCtr_ == ucNoOfCfgDataFields_)
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

/**
*************************************************************************************************************
*	\brief	NMEA Protocol parser function
*
*	Function parses the received NMEA sentence. All supported NMEA frames are configured in a
*	configuration table. The Parser uses this table to find the functions which convert a certain
*	data field and the variable which stores the result of this conversion. Within the
*	configuration table a combination of a datatype and a target variable can be found for
*	each data element in a NMEA frame. The datatype is connected with a coversion function.
*	
*	\return	
*		void
*
*	\param
*		pucMessage_		Pointer to the received NMEA message in the DLL buffer
*
*	\param
*		ucFrameNo_		Index of the frame in the configuration table
*	
*************************************************************************************************************
*/
void vParseFrame(uchar8 *pucMessage_, uchar8 ucFrameNo_)
{
	const				tNMEAFrameFormat *psFrameCfgEntry_;
	uchar8				ucNoOfDataFields_;
	uchar8				ucDataFieldIdx_;
	uchar8				ucFrameCharacterIdx_;
	tNMEADataType		eDatatype_;
	tfpConvFunction		fpConverter_;
	void*				pTargetVar_;

	ucFrameCharacterIdx_ = 0;
	
	/* Read pointer to the NMEA frame configuration */
	psFrameCfgEntry_ = (tNMEAFrameFormat*)pgm_read_word(&NCFG_asSupportedFrames[ucFrameNo_]);

	ucNoOfDataFields_ = (uchar8)pgm_read_byte(&(psFrameCfgEntry_->ucNoOfDataFields));

	/* Start to parse each data entry after one other */
	for(ucDataFieldIdx_ = 0; ucDataFieldIdx_ < ucNoOfDataFields_; ucDataFieldIdx_++)
	{
		/* Search beginning of a data field (data fields always begin with ',') */
		while(pucMessage_[ucFrameCharacterIdx_] != NPAR_DATA_FIELD_SEPARATOR)
		{
			ucFrameCharacterIdx_++;
		}

		/* Set index on the first character after ',' */
		ucFrameCharacterIdx_++;

		/* Check if datafield needs to be converted */
		eDatatype_ = (tNMEADataType)pgm_read_byte(&(psFrameCfgEntry_->sDataConnection[ucDataFieldIdx_].eDataType));

		if(eDatatype_ != NCONV_TypeUndefined)
		{
			/* Call conversion function to handle data field */
			fpConverter_ = (tfpConvFunction)pgm_read_word(&(NCONV_fpConvFunctionTable[eDatatype_]));
			pTargetVar_ = 
				(void *)pgm_read_word(&(psFrameCfgEntry_->sDataConnection[ucDataFieldIdx_].pTargetVar));
			(void)fpConverter_(&pucMessage_[ucFrameCharacterIdx_], pTargetVar_);
		}
	}
}

/*
*************************************************************************************************************
*   GLOBAL FUNCTIONS
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   void NPAR_vHandleTask(void)
*************************************************************************************************************
*/
void NPAR_vHandleTask(void)
{
	uchar8 *pucMessage_;
	uchar8 ucFrameNo_;

	if(NDLL_ucIsFrameInBuffer() == TRUE)
	{
		/* fetch received NMEA frame */
		pucMessage_ = NDLL_pucGetReceivedNMEAFrame();
		
		ucFrameNo_ = ucIsFrameSupported(pucMessage_);

		if(ucFrameNo_ != NPAR_FRAME_NOT_SUPPORTED)
		{
			if(bCheckNMEAFrame(pucMessage_, ucFrameNo_) == TRUE)
			{
				/* Frame has been found in the configuration table
					and corresponds to the configuration
					=> start parsing it */
				vParseFrame(pucMessage_, ucFrameNo_);
			}
		}

		/* Release frame from buffer after it has been processed */
		NDLL_vReleaseFrameFromBuffer();
	}
}

