/**
*************************************************************************************************************
*
*   \file timer.c
*
*   Software timer module
*
*   This module implements a software timer module. These timers can
*	be used by other software modules to generate defined times which
*	are derived from a basic timetick. This timetick is usually
*	generated by a hardware timer interrupt of the micrcontroller.
*	The timers which are managed by this module are basically
*	downcounters. The timers itself are located in the application
*	modules.\n
*	Before a certain software timer can be handled by this module, it
*	has to be created by the application module. When the timer is
*	created, it is entered in a linked list. This module can only
*	handle timers which are located in the list.\n
*   \n
*   \b Shortcut:    TMR
*
*   \author         Ralf Hochhausen
*   \date           08.06.2006
*   \version        1.0
*
*   (c) 2006 by Ralf Hochhausen (e-mail: micro@ralf-hochhausen.de)
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
*   \b History:
*
*   08.06.2006      Ralf Hochhausen\n
*                   Revision 1.0\n
*                   Initial Revision
*
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   INCLUDES 
*************************************************************************************************************
*/
#include "types.h"				/* Include generic type definitions */
#include "global.h"				/* Include globale makros and definitions */
#include "systime.h"			/* Include system time module */
#include "timer.h"				/* Include own header */

/*
*************************************************************************************************************
*   LOCAL DEFINITIIONS
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   LOCAL MACROS
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   LOCAL TYPES
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   LOCAL CONSTANTS
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   STATIC VARIABLES
*************************************************************************************************************
*/
static tEventTimer *pTimerList		= NULL;			/**< Pointer to the first registered software
														 timer in the list */

/*
*************************************************************************************************************
*   LOCAL FUNCTION PROTOTYPES
*************************************************************************************************************
*/
void vHandleTimers(void);

/*
*************************************************************************************************************
*   LOCAL FUNCTIONS
*************************************************************************************************************
*/

/**
*************************************************************************************************************
*
*   \brief		Timer interrupt handler function.
*
*				Function is registered in the system timer module as the basetimer
*				tick event handler. The function counts down all software timers
*				that are loaded with some timer value.
*
*   \return		void
*
*	\remark		The function does not call any registered event handler function
*   
*************************************************************************************************************
*/
void vHandleTimers(void)
{
	tEventTimer	*pTmpTimer_;

	pTmpTimer_ = pTimerList;

	/* Decrement all timers in the event timer list */
	while(pTmpTimer_ != NULL)
	{
		if(pTmpTimer_->uiTimer)
		{
			pTmpTimer_->uiTimer--;
		}

		pTmpTimer_ = pTmpTimer_->pNextTimer;
	}
}

/*
*************************************************************************************************************
*   GLOBAL FUNCTIONS
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*	void TMR_vCreateEventTimer(tEventTimer *pTimer_)
*************************************************************************************************************
*/
void TMR_vCreateEventTimer(tEventTimer *pTimer_)
{
	tEventTimer	*pTmpTimer_;
	
	pTimer_->uiTimer	= 0;			/* Reset timer */
	pTimer_->pEvent		= NULL;			/* Reset event function */
	pTimer_->eState		= TMR_STOPPED;	/* After creation the timer is stopped */
	pTimer_->pNextTimer	= NULL;			/* New timers are inserted at the end of the list */

	/* Insert timer into the list */
	if(pTimerList != NULL)
	{
		/* Search end of the list */
		pTmpTimer_ = pTimerList;
		while(pTmpTimer_->pNextTimer != NULL)
		{
			pTmpTimer_ = pTmpTimer_->pNextTimer;
		}

		vEnterCritical();		/* Disable interrupts */
		pTmpTimer_->pNextTimer = pTimer_;
		vExitCritical();		/* Reenable interrupts */
	}
	else
	{
		vEnterCritical();		/* Disable interrupts */
		/* First timer of the list */
		pTimerList = pTimer_;
		vExitCritical();		/* Reenable interrupts */
	}
}

/*
*************************************************************************************************************
*	void TMR_vLoadEventTimer(tEventTimer *pTimer_, uint16 uiTime_, TimerEvent pEvent)
*************************************************************************************************************
*/
void TMR_vLoadEventTimer(tEventTimer *pTimer_, uint16 uiTime_, TimerEvent pEvent)
{
	vEnterCritical();
	pTimer_->uiTimer	= uiTime_;		/* Load timer */
	pTimer_->pEvent		= pEvent;		/* Set event function pointer */
	pTimer_->eState		= TMR_RUNNING;	/* Timer is activated now */
	vExitCritical();
}

/*
*************************************************************************************************************
*   void TMR_vHandleTask(void)
*************************************************************************************************************
*/
void TMR_vHandleTask(void)
{
	tEventTimer	*pTmpTimer_;

	/* Scan timer list for event functions that have to be called */
	pTmpTimer_ = pTimerList;

	while(pTmpTimer_ != NULL)
	{
		if(pTmpTimer_->eState == TMR_RUNNING)
		{
			/* Timer is active -> check if timer is down */
			if(pTmpTimer_->uiTimer == 0)
			{
				vEnterCritical();
				pTmpTimer_->eState = TMR_STOPPED;	/* Stop timer */
				vExitCritical();

				if(pTmpTimer_->pEvent != NULL)
				{	
					/* Call event function */
					pTmpTimer_->pEvent();
				}
			}
		}

		pTmpTimer_ = pTmpTimer_->pNextTimer;	/* Set pointer to the next timer */
	}
}

/*
*************************************************************************************************************
*   void TMR_vInit(void)
*************************************************************************************************************
*/
void TMR_vInit(void)
{
	/* Register SW timer handler function in the system timer module */
	SYST_vRegisterTickHandler(vHandleTimers);
}
