/**
*************************************************************************************************************
*
*   \file timer.h
*
*   Software timer module
*
*   This module implements a software timer module. These timers can
*	be used by other software modules to generate defined times which
*	are derived from a basic timetick. This timetick is usually
*	generated by a hardware timer interrupt of the micrcontroller.
*	The timers which are managed by this module are basically
*	downcounters. The timers itself are located in the application
*	modules.\n
*	Before a certain software timer can be handled by this module, it
*	has to be created by the application module. When the timer is
*	created, it is entered in a linked list. This module can only
*	handle timers which are located in the list.\n
*   \n
*   \b Shortcut:    TMR
*
*   \author         Ralf Hochhausen
*   \date           08.06.2006
*   \version        1.0
*
*   (c) 2006 by Ralf Hochhausen (e-mail: micro@ralf-hochhausen.de)
*
*   This program is free software; you can redistribute it and/or
*   modify it under the terms of the GNU General Public License
*   as published by the Free Software Foundation; either version 2
*   of the License, or (at your option) any later version.
*   
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
*   \b History:
*
*   08.06.2006      Ralf Hochhausen\n
*                   Revision 1.0\n
*                   Initial Revision
*
*************************************************************************************************************
*/

#ifndef __TMR_H
#define __TMR_H

/*
*************************************************************************************************************
*   INCLUDES 
*************************************************************************************************************
*/
#include "types.h"

/*
*************************************************************************************************************
*   GLOBAL DEFINITIIONS
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   GLOBAL MACROS
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   GLOBAL TYPES
*************************************************************************************************************
*/

/**
*	Typedefintion of the actual timer state
*	Timers may be running or stopped. Event functions are only called when
*	the timer is running.
*/
typedef enum
{
	TMR_RUNNING,		/**< Timer is running */
	TMR_STOPPED			/**< Timer is stopped */
}tTmrState;

/**
*	Type definition of the timer event function
*	Functions of this type are used as timer call back functions.
*	These functions are called when a timer ran down to zero
*/
typedef void (* TimerEvent)(void);

/**
*	Type defintion of the timer objects
*	All software timers which are handled by this module have the following structure.
*	Timers are organized within a linked list.
*/
typedef struct Timer
{	
	uint16			uiTimer;			/**< SW timer counter value */
	TimerEvent		pEvent;				/**< Pointer to the timer callback event function */
	struct Timer	*pNextTimer;		/**< Pointer to the next timer in the list */
	tTmrState		eState;				/**< Actual state of the timer */
}tEventTimer;

/*
*************************************************************************************************************
*   GLOBAL CONSTANTS
*************************************************************************************************************
*/

/*
*************************************************************************************************************
*   GLOBAL FUNCTIONS
*************************************************************************************************************
*/

/**
*************************************************************************************************************
*
*   \brief		Create software timer function
*
*				Function is used to insert a software timer in the timer list of the
*				module.	
*
*	\param[in]	*pTimer_	Pointer to the timer to be inserted
*
*   \return		void
*
*	\remark		All software timers have to be created before they can be used by this module
*   
*************************************************************************************************************
*/
void TMR_vCreateEventTimer(tEventTimer *pTimer_);

/**
*************************************************************************************************************
*
*   \brief		Load event timer function.
*
*				Function is used to start or reload any event timer. When the timer
*				was stopped until now it gets started.
*
*	\param[in]	*pTimer_	Pointer to the software timer to be loaded
*	\param[in]	uiTime_		Time to be loaded in [no. of baseticks]
*	\param[in]	pEvent		Pointer to the event callback function
*
*   \return		void
*   
*************************************************************************************************************
*/
void TMR_vLoadEventTimer(tEventTimer *pTimer_, uint16 uiTime_, TimerEvent pEvent);

/**
*************************************************************************************************************
*
*   \brief		Task handler function of the module.
*
*				Function scans the whole software timer list for timers
*				that have been counted down to zero. When a timer is down, the
*				corresponding event function is called if there is any present.
*
*   \return		void
*   
*************************************************************************************************************
*/
void TMR_vHandleTask(void);

/**
*************************************************************************************************************
*
*   \brief Initialization of the software timer module
*
*	Function Initializes the internal data structures. Function has to
*	be called before the module can be used.
*
*   \return		void
*   
*************************************************************************************************************
*/
void TMR_vInit(void);

#endif  /* __TMR_H */
